=== modified file 'utilities/local-latency'
--- utilities/local-latency	2011-08-04 19:26:11 +0000
+++ utilities/local-latency	2011-08-04 20:26:07 +0000
@@ -2,13 +2,14 @@
 
 __metaclass__ = type
 
-from optparse import OptionParser
 import sys
 import subprocess
 
 from script_commands import (
     Command,
     UserError,
+    types,
+    helps,
     )
 
 
@@ -16,43 +17,26 @@
      subprocess.call('sudo tc ' + command, shell=True)
 
 
-class StartCommand(Command):
-
-    @classmethod
-    def get_parser(cls):
-        parser = OptionParser()
-        parser.add_option(
-            '-d', '--delay', dest='delay', type='int',
-            help='Length of delay in miliseconds (each way).'
-            ' Default: %default.')
-        return parser
-
-    @staticmethod
-    def run(delay=500, port=443):
-        tc('qdisc add dev lo root handle 1: prio')
-        tc('qdisc add dev lo parent 1:3 handle 30: netem delay %dms' % delay)
-        tc('filter add dev lo protocol ip parent 1:0 prio 3 u32 match ip'
-           ' dport %d 0xffff flowid 1:3' % port)
-        tc('filter add dev lo protocol ip parent 1:0 prio 3 u32 match ip'
-           ' sport %d 0xffff flowid 1:3' % port)
-
-
-Command.commands['start'] = StartCommand
-
-
-class StopCommand(Command):
-
-    @staticmethod
-    def get_parser():
-        parser = OptionParser()
-        return parser
-
-    @staticmethod
-    def run():
-        tc('qdisc del dev lo root')
-
-
-Command.commands['stop'] = StopCommand
+@types(delay='int', port='int')
+@helps(delay='Length of delay in miliseconds (each way).',
+       port='Port to induce delay on.')
+def start(delay=500, port=443):
+    tc('qdisc add dev lo root handle 1: prio')
+    tc('qdisc add dev lo parent 1:3 handle 30: netem delay %dms' % delay)
+    tc('filter add dev lo protocol ip parent 1:0 prio 3 u32 match ip'
+       ' dport %d 0xffff flowid 1:3' % port)
+    tc('filter add dev lo protocol ip parent 1:0 prio 3 u32 match ip'
+       ' sport %d 0xffff flowid 1:3' % port)
+
+
+Command.commands['start'] = start
+
+
+def stop():
+    tc('qdisc del dev lo root')
+
+
+Command.commands['stop'] = stop
 
 
 

=== modified file 'utilities/script_commands.py'
--- utilities/script_commands.py	2011-08-04 19:19:42 +0000
+++ utilities/script_commands.py	2011-08-04 20:26:07 +0000
@@ -1,40 +1,64 @@
 import inspect
+from optparse import OptionParser
 
 
 class UserError(Exception):
     pass
 
 
+def add_dict(name, **kwargs):
+    def decorator(func):
+        setattr(func, name, kwargs)
+        return func
+    return decorator
+
+
+def types(**kwargs):
+    return add_dict('_types', **kwargs)
+
+def helps(**kwargs):
+    return add_dict('_helps', **kwargs)
+
+
+
+def get_function_parser(function):
+    parser = OptionParser()
+    args, ignore, ignored, defaults = inspect.getargspec(function)
+    for arg in args:
+        arg_type = function._types.get(arg)
+        if arg_type is None:
+            continue
+        arg_help = getattr(function, '_helps', {}).get(arg)
+        if arg_help is not None:
+            arg_help +=' Default: %default.'
+        parser.add_option('--%s' % arg, type=arg_type, help=arg_help)
+    if defaults is not None:
+        defaults_dict = dict(zip(args, defaults))
+        option_defaults = dict(
+            (key, value) for key, value in defaults_dict.items()
+            if parser.defaults.get(key, '') is None)
+        parser.set_defaults(**option_defaults)
+    return parser
+
+
 class Command:
     """Base class for subcommands."""
 
     commands = {}
 
     @classmethod
-    def parse_args(cls, args):
+    def parse_args(cls, command, args):
         if len(args) != 0:
             raise UserError('Too many arguments.')
         return {}
 
     @classmethod
-    def set_defaults(cls, parser):
-        args, ignored, ignored, defaults = inspect.getargspec(cls.run)
-        if defaults is None:
-            return
-        defaults_dict = dict(zip(args, defaults))
-        option_defaults = dict(
-            (key, value) for key, value in defaults_dict.items()
-            if parser.defaults.get(key, '') is None)
-        parser.set_defaults(**option_defaults)
-
-    @classmethod
-    def run_from_args(cls, cmd_args):
-        parser = cls.get_parser()
-        cls.set_defaults(parser)
+    def run_from_args(cls, command, cmd_args):
+        parser = get_function_parser(command)
         options, args = parser.parse_args(cmd_args)
-        kwargs = cls.parse_args(args)
+        kwargs = cls.parse_args(command, args)
         kwargs.update(options.__dict__)
-        cls.run(**kwargs)
+        command(**kwargs)
 
     @classmethod
     def run_subcommand(cls, argv):
@@ -46,4 +70,4 @@
         except KeyError:
             raise UserError('%s invalid.  Valid commands: %s.' %
                             (argv[0], ', '.join(cls.commands.keys())))
-        command.run_from_args(argv[1:])
+        cls.run_from_args(command, argv[1:])

